/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
                   y
                   |
                   |
                   | v[4]
                 1 +-----------------+ v[5]=(1,1,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v[1]               |
           |  v[0] +---------|-------+------> x           |
           |      /          |      /1                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         1 +-----------------+                         /
         / v[3]=(0,0,1)      v[2]=(1,0,1)            z
       /
     /
    z

   Render a single wireframe cube.

   This version creates LineSegment objects that do not share any
   Vertex objects. So each corner of the cube gets represented by
   three different Vertex objects, one in each LineSegment that meets
   at that corner. And since the three LineSegment objects that meet
   at a corner don't share a Vertex object, the three LineSegment
   objects can have three different colors assiged to them.
*/
public class InteractiveCube_v2a
{
   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the cube along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the cube.");
      System.out.println("Use the 'c' key to change the random EDGE colors.");

      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // Add geometry to the Model.
      final Vertex[] v = new Vertex[8];

      v[0] = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      v[1] = new Vertex(1.0, 0.0, 0.0);
      v[2] = new Vertex(1.0, 0.0, 1.0);
      v[3] = new Vertex(0.0, 0.0, 1.0);
      v[4] = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      v[5] = new Vertex(1.0, 1.0, 0.0);
      v[6] = new Vertex(1.0, 1.0, 1.0);
      v[7] = new Vertex(0.0, 1.0, 1.0);

      model.addLineSegment( new LineSegment(new Vertex(v[0]), new Vertex(v[1])) );  // bottom face
      model.addLineSegment( new LineSegment(new Vertex(v[1]), new Vertex(v[2])) );  // bottom face
      model.addLineSegment( new LineSegment(new Vertex(v[2]), new Vertex(v[3])) );  // bottom face
      model.addLineSegment( new LineSegment(new Vertex(v[3]), new Vertex(v[0])) );  // bottom face
      model.addLineSegment( new LineSegment(new Vertex(v[4]), new Vertex(v[5])) );  // top face
      model.addLineSegment( new LineSegment(new Vertex(v[5]), new Vertex(v[6])) );  // top face
      model.addLineSegment( new LineSegment(new Vertex(v[6]), new Vertex(v[7])) );  // top face
      model.addLineSegment( new LineSegment(new Vertex(v[7]), new Vertex(v[4])) );  // top face
      model.addLineSegment( new LineSegment(new Vertex(v[0]), new Vertex(v[4])) );  // back face
      model.addLineSegment( new LineSegment(new Vertex(v[1]), new Vertex(v[5])) );  // back face
      model.addLineSegment( new LineSegment(new Vertex(v[3]), new Vertex(v[7])) );  // front face
      model.addLineSegment( new LineSegment(new Vertex(v[2]), new Vertex(v[6])) );  // front face

      // Give the edges of the model random colors.
      for (LineSegment ls : model.lineSegmentList)
      {
         ls.setColorRandom();
      }

      // Push the model away from where the camera is.
      for (LineSegment ls : model.lineSegmentList)
      {
         ls.v[0].z -= 2;
         ls.v[1].z -= 2;
      }


      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 2", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Rasterize.debug = ! Rasterize.debug;
            }
            else if ('c' == c)
            {
               // Change the random color at each edge.
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.setColorRandom();
               }
            }
            else if ('p' == c)
            {
               scene.camera.perspective = ! scene.camera.perspective;
            }
            else if ('s' == c) // Scale the cube 10% smaller.
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].x *= 0.9;  // this is NOT correct
                  ls.v[0].y *= 0.9;
                  ls.v[0].z *= 0.9;
                  ls.v[1].x *= 0.9;
                  ls.v[1].y *= 0.9;
                  ls.v[1].z *= 0.9;
               }
            }
            else if ('S' == c) // Scale the cube 10% larger.
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].x *= 1.1;  // this is NOT correct
                  ls.v[0].y *= 1.1;
                  ls.v[0].z *= 1.1;
                  ls.v[1].x *= 1.1;
                  ls.v[1].y *= 1.1;
                  ls.v[1].z *= 1.1;
               }
            }
            else if ('x' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].x += 0.1;
                  ls.v[1].x += 0.1;
               }
            }
            else if ('X' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].x -= 0.1;
                  ls.v[1].x -= 0.1;
               }
            }
            else if ('y' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].y += 0.1;
                  ls.v[1].y += 0.1;
               }
            }
            else if ('Y' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].y -= 0.1;
                  ls.v[1].y -= 0.1;
               }
            }
            else if ('z' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].z += 0.1;
                  ls.v[1].z += 0.1;
               }
            }
            else if ('Z' == c)
            {
               for (LineSegment ls : model.lineSegmentList)
               {
                  ls.v[0].z -= 0.1;
                  ls.v[1].z -= 0.1;
               }
            }
            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            //System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
