/*

*/
package pipeline;
import scene.*;
import framebuffer.*;

/**
    This renderer takes as its input a Scene data structure and
    a FrameBuffer data structure. This renderer mutates the
    FrameBuffer's current viewport so that it is filled in with
    the rendered image of the scene represented by the Scene object.
<p>
    This implements our second rendering pipeline. It is essentially
    the same as the previous renderer but it adds color to the vertices.
*/
public class Pipeline
{
   public static boolean debug = false;

   /**
      Mutate the FrameBuffer's current viewport so that it holds
      the rendered image of the Scene object.

      @param scene Scene object to render
      @param fb    FrameBuffer to hold rendered image of the scene
   */
   public static void render(Scene scene, FrameBuffer fb)
   {
      if ( debug )
         render_debug(scene, fb);
      else
         render_(scene, fb);
   }


   /**
      Renderer without any debugging information.

      @param scene Scene object to render
      @param fb    FrameBuffer to hold rendered image of the scene
   */
   private static void render_(Scene scene, FrameBuffer fb)
   {
      // For every Model in the Scene, render
      // every LineSgement in the Model.
      for (Model model : scene.modelList)
      {
         for (LineSegment ls : model.lineSegmentList)
         {
            // Make a deep copy of the line segment.
            LineSegment ls2 = new LineSegment(ls);

            // 1. Apply the projection transformation.
            Project.project(ls2, scene.camera);

            // 2. Apply the viewport transformation.
            Viewport.viewport(ls2, fb);

            // 3. Rasterize the line segment into pixels.
            Rasterize.rasterize(ls2, fb);
         }
      }
   }



   /**
      Render with quite a bit of debugging information.

      @param scene Scene object to render
      @param fb    FrameBuffer to hold rendered image of the scene
   */
   private static void render_debug(Scene scene, FrameBuffer fb)
   {
      System.out.println("=== Render Scene ===================================");

      // For every Model in the Scene, render
      // every LineSgement in the Model.
      for (Model model : scene.modelList)
      {
         System.out.println("=== Render Model ===================================");

         for (LineSegment ls : model.lineSegmentList)
         {
            // Make a deep copy of the line segment.
            LineSegment ls2 = new LineSegment(ls);

            // log interesting information to standard output
            logLineSegment("0. Input", ls2);

            // 1. Apply the projection transformation.
            Project.project(ls2, scene.camera);

            // log interesting information to standard output
            logLineSegment("1. Projected", ls2);

            // 2. Apply the viewport transformation.
            Viewport.viewport(ls2, fb);

            // log interesting information to standard output
            logLineSegment("2. Viewport", ls2);

            // 3. Rasterize the line segment into pixels.
            Rasterize.rasterize(ls2, fb);

            System.out.println();
         }
         System.out.println("=== End Model ===================================");
      }
      System.out.println("=== End Scene ===================================");
   }


   private static void logLineSegment(String stage, LineSegment ls)
   {
      System.out.print( stage + " " + ls.toString() );
   }
}
