/*

*/
package pipeline;
import scene.*;

/**
<pre>
   Project line segments from camera coordinates to the view plane.

   For each Vertex object in each LineSgement object, project the
   Vertex object into the view plane z = -1.

   Let us derive the formulas for the perspective projection transformation
   (the formulas for the parallel projection transformation are pretty obvious).
   We will derive the x-coordinate formula; the y-coordinate formula is similar.

   Let (x_c, y_c, z_c) denote a point in the 3-dimensional camera coordinate
   system. Let (x_p, y_p, -1) denote the point's perspective projection into
   the view plane, z = -1. Here is a "picture" of just the (x, z)-plane in
   camera space. This picture shows the point (x_c, z_c) and its projection to
   the point (x_p, -1) in the view plane.

           x
           |                             /
           |                           /
       x_c +                         + (x_c, z_c)
           |                       / |
           |                     /   |
           |                   /     |
           |                 /       |
           |               /         |
           |             /           |
       x_p +           +             |
           |         / |             |
           |       /   |             |
           |     /     |             |
           |   /       |             |
           | /         |             |
           +-----------+-------------+------------> -z
        (0,0)         -1            z_c

   We are looking for a formula that computes x_p in terms of x_c and z_c.
   There are two similar triangles in this picture that share a vertex at the
   origin. Using the properties of similar triangles we have the following ratios.
   (Remember that these are ratios of positive lengths, so we write -z_c, since
    z_c is on the negative z-axis).

                 x_p       x_c
                -----  =  -----
                  1       -z_c

   If we solve this ratio for the unknown, x_p, we get the projection formula,

                 x_p = -x_c / z_c.

   The equivalent formula for the y-coordinate is

                 y_p = -y_c / z_c.
</pre>
*/
public class Project
{
   /**
      For each line segment in the model, project
      each vertex to the view plane z = -1.

      @param ls LineSegment to project onto the image plane
      @param camera a reference to the Scene's Camera object
   */
   public static void project(LineSegment ls, Camera camera)
   {
      // For each vertex in the line segment,
      // compute its projected (x,y) coordinates.
      double v0_xp, v0_yp, v1_xp, v1_yp;

      if ( camera.perspective )
      {
         // calculate the perspective projection
         ls.v[0].x = ls.v[0].x / -ls.v[0].z;  // xp = xc / -zc
         ls.v[0].y = ls.v[0].y / -ls.v[0].z;  // yp = yc / -zc
         ls.v[1].x = ls.v[1].x / -ls.v[1].z;
         ls.v[1].y = ls.v[1].y / -ls.v[1].z;
      }
      else
      {
         // calculate the parallel projection instead
         ls.v[0].x = ls.v[0].x;
         ls.v[0].y = ls.v[0].y;
         ls.v[1].x = ls.v[1].x;
         ls.v[1].y = ls.v[1].y;
      }
   }
}
