/*

*/
package pipeline;
import scene.*;
import framebuffer.*;

/**
<pre>
   Transform each line segment's endpoints from view plane coordinates
   to viewport coordinates so that the view rectangle in the view plane
   with
        -1 <= x <= 1,
        -1 <= y <= 1,
   transforms into a viewport where
       0.5 <= x < w + 0.5,
       0.5 <= y < h + 0.5,
   where w = number of horizontal pixels in the viewport,
         h = number of vertical pixels in the viewport.

   The goal of this transformation is to put a logical pixel with
   integer coordinates at the center of each square physical pixel.
   The logical pixel with integer coordinates (m, n) represents the
   square pixel with
     m - 0.5 <= x < m + 0.5,
     n - 0.5 <= y < n + 0.5.
   Notice that logical pixel integer coordinates (m.n) have
     1 <= m <= w
     1 <= n <= h.

   Let us derive the formulas for the viewport transformation (we will
   derive the x-coordinate formula; the y-coordinate formula is similar).

   Let x_p denote an x-coordinate in the view plane and let x_vp denote
   an x-coordinate in the viewport. If a vertex is on the left edge of
   the view rectangle (with x_p = -1), then it should be transformed to
   the left edge of the viewport (with x_vp = 0.5). And if a vertex is on
   the right edge of the view rectangle (with x_p = 1), then it should be
   transformed to the right edge of the viewport (with x_vp =  w + 0.5).
   These two facts are all we need to know to find the linear function
   for the transformation of the x-coordinate.

   We need to calculate the slope m and intercept b of a linear function
             x_vp = m * x_p + b
   that converts view plane coordinates into viewport coordinates. We know,
   from what we said above about the left and right edges of the view
   rectangle, that
              0.5 = (m * -1) + b,
          w + 0.5 = (m *  1) + b.
   If we add these last two equations together we get
            w + 1 = 2*b
   or
            b = (w + 1)/2.
   If we use b to solve for m we have
              0.5 = (m * -1) + (w + 1)/2
                1 = -2*m + w + 1
              2*m = w
                m = w/2.
   So the linear transformation of the x-coordinate is
          x_vp = (w/2) * x_p + (w+1)/2
               = 0.5 + w/2 * (x_p + 1).

   The equivalent formula for the y-coordinate is
          y_vp = 0.5 + h/2 * (y_p + 1).
</pre>
*/
public class Viewport
{
   /**
      Transform the line segment's endpoints from
      view plane coordinates to viewport coordinates.

      @param ls LineSegment to transform to viewport coordinates
      @param fb FrameBuffer that holds the current viewport
   */
   public static void viewport(LineSegment ls, FrameBuffer fb)
   {
      // Get the viewport dimensions.
      int w = fb.getWidthVP();
      int h = fb.getHeightVP();

      // Transform the line segment's endpoints from
      // view plane coordinates to viewport coordinates.
      ls.v[0].x = 0.5 + w/2.001 * (ls.v[0].x + 1);//x_vp = 0.5 + w/2 * (x_p+1)
      ls.v[0].y = 0.5 + h/2.001 * (ls.v[0].y + 1);//y_vp = 0.5 + h/2 * (y_p+1)
      ls.v[1].x = 0.5 + w/2.001 * (ls.v[1].x + 1);
      ls.v[1].y = 0.5 + h/2.001 * (ls.v[1].y + 1);
      // NOTE: Notice the 2.001 fudge factor in the last four equations.
      // This is explained on page 142 of
      //    "Jim Blinn's Corner: A Trip Down The Graphics Pipeline"
      //     by Jim Blinn, 1996, Morgan Kaufmann Publishers.
   }
}
