/*

*/
package scene;

import java.util.List;
import java.util.LinkedList;
import java.awt.Color;

/**
    A Model data structure represents a distinct geometric object in a Scene.
    A Model data structure is mainly a list of LineSegment objects. Each
    LineSegment object contains two Vertex objects with coordinates in the
    camera coordinate system.
<p>
    The line segments represent the geometric object as a "wire-frame", that is,
    the geometric object is drawn as a collection of "edges". This is a fairly
    simplistic way of doing 3D graphics and we will improve this in later renderers.
<p>
    See
<br>    http://en.wikipedia.org/wiki/Wire-frame_model
    or
<br>    https://www.google.com/search?q=graphics+wireframe&tbm=isch
*/
public class Model
{
   public List<LineSegment> lineSegmentList = new LinkedList<LineSegment>();

   /**
      Construct an empty model.
   */
   public Model()
   {
   }


   /**
      Add a LineSegment (or LineSegments) to this model's list of line segments.

      @param lsArray array of LineSegments to add to this Model
   */
   public void addLineSegment(LineSegment... lsArray)
   {
      for (LineSegment ls : lsArray)
      {
         this.lineSegmentList.add(ls);
      }
   }


   /**
      Set all the line segments in this model to the same color.

      @param c Color for all of this model's LineSegments
   */
   public void setColor(Color c)
   {
      for (LineSegment ls : this.lineSegmentList)
      {
         ls.setColor(c);
      }
   }

   /**
      Set all the line segments in this model to the same random color.
   */
   public void setColorRandom()
   {
      lineSegmentList.get(0).setColorRandom();
      Color c = lineSegmentList.get(0).v[0].getColor();
      for (LineSegment ls : this.lineSegmentList)
      {
         ls.setColor(c);
      }
   }

   /**
      Set all the line segments in this model to random colors.
   */
   public void setRandomColors()
   {
      for (LineSegment ls : this.lineSegmentList)
      {
         ls.setColorRandom();
      }
   }


   /**
      For debugging.

      @return String representation of this Model object
   */
   public String toString()
   {
      String result = "";
      result += "This Model has " + lineSegmentList.size() + " line segments\n";
      //result = "Printing out this Model's " + lineSegmentList.size() + " Line segments:\n";
      for (LineSegment ls : this.lineSegmentList)
      {
         result += ls.toString();
      }
      //result += "Done printing out Model\n";
      return result;
   }
}
